from pl_user_io.open_url import open_url
from pl_user_io.pause_and_read import pause_and_read
from pl_user_io.wait_for_enter import wait_for_enter


def task_with_url(instruction: str, url_to_open: str) -> None:
    pause_and_read(instruction)
    # I ran into an issue on mac startup where this was trying to open chrome, not firefox.
    # I fixed it by starting firefox on device startup.
    open_url(url_to_open)
    wait_for_enter()



from pl_user_io.testing.assertions import (
    assert_asked_yes_no,
    assert_asked_yes_no_exactly_once,
    assert_loading_spinner_displayed,
    assert_not_asked_yes_no,
    assert_task_not_performed,
    assert_task_performed,
)
from pl_user_io.testing.stubs import (
    input_count,
    stub_str_input,
    stub_str_input_callback,
    stub_yes_no,
)
from pl_user_io.testing.user_io_fake import (
    InputMappingCallback,
    UserIOFake,
    assert_displayed,
    assert_displayed_exactly_once,
    assert_displayed_in_order,
    assert_not_displayed,
    user_io_fake,
)

__all__ = [
    "InputMappingCallback",
    "UserIOFake",
    "assert_asked_yes_no",
    "assert_asked_yes_no_exactly_once",
    "assert_displayed",
    "assert_displayed_exactly_once",
    "assert_displayed_in_order",
    "assert_loading_spinner_displayed",
    "assert_not_asked_yes_no",
    "assert_not_displayed",
    "assert_task_not_performed",
    "assert_task_performed",
    "input_count",
    "stub_str_input",
    "stub_str_input_callback",
    "stub_yes_no",
    "user_io_fake",
]



from pl_user_io.assert_no import assert_no
from pl_user_io.assert_yes import assert_yes
from pl_user_io.delay import delay
from pl_user_io.display import display
from pl_user_io.loading_spinner import loading_spinner
from pl_user_io.open_url import open_url
from pl_user_io.pause_and_read import pause_and_read
from pl_user_io.pause_to_read import pause_to_read
from pl_user_io.str_input import str_input
from pl_user_io.task import task
from pl_user_io.task_with_url import task_with_url
from pl_user_io.wait_for_enter import wait_for_enter
from pl_user_io.yes_no import yes_no

__all__ = [
    "assert_no",
    "assert_yes",
    "delay",
    "display",
    "loading_spinner",
    "open_url",
    "pause_and_read",
    "pause_to_read",
    "str_input",
    "task",
    "task_with_url",
    "wait_for_enter",
    "yes_no",
]



from unittest.mock import Mock

from pl_user_io.str_input import StrInputResponse
from pl_user_io.testing.user_io_fake import InputMappingCallback, user_io_fake

StrInputMock = Mock


def stub_str_input_callback(callback: InputMappingCallback) -> None:
    user_io_fake().input_mapping_callbacks.insert(0, callback)


def stub_str_input(response: StrInputResponse, prior_output: str) -> None:
    def _callback(output: str) -> StrInputResponse | None:
        if output.endswith(prior_output):
            return response
        return None

    user_io_fake().input_mapping_callbacks.insert(0, _callback)


def input_count(mock: StrInputMock) -> int:
    return mock.call_count


def stub_yes_no(question: str, answer: bool) -> None:
    stub_str_input("y" if answer else "n", f"{question} (y/n)")



from pl_mocks_and_fakes import (
    MockInUnitTests,
    MockReason,
)


@MockInUnitTests(MockReason.UNMITIGATED_SIDE_EFFECT)
def display(*values: object, end: str | None = "\n", flush: bool = False) -> None:
    print(*values, end=end, flush=flush)
